#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This module contains the julia fractal class.
'''

# Python modules
import gtk
import gtk.gdk
import math

# Application modules
from fractal import *

class Julia(Fractal):
    '''
        The Julia set fractal.
    '''

    def __init__(self, app):
        '''
            Constructor. Initializes the fractal calculation.
        '''
        Fractal.__init__(self, app)
        self.const = -0.8 + 0j
        self.power = 2
        self.maxIter = 50
        self.defaultTopLeft = -1.8 -1.5j
        self.defaultBottomRight = 1.8 + 1.5j

    def parameter_dialog(self):
        '''
            This method opens a GTK dialog and prompts the user for julia set
            specific parameters.
        '''
        # Get object references
        self.gladeFile  = self.app.gladeFile
        self.wTree      = gtk.glade.XML(self.gladeFile, root="dlgJuliaParam")
        self.dlg        = self.wTree.get_widget("dlgJuliaParam")
        self.entConst   = self.wTree.get_widget("entJuliaParamConstant")
        self.entPower   = self.wTree.get_widget("entJuliaParamPower")
        self.entMaxIter = self.wTree.get_widget("entJuliaParamMaxIter")
        self.entBail    = self.wTree.get_widget("entJuliaParamBail")
        
        self.dlg.connect("delete-event", self.dlg.hide_on_delete)
        self.dlg.set_icon_from_file(self.app.iconFile)
        
        # Set default values and display
        self.entConst.set_text(str(self.const).replace('(', '').replace(')', '').replace('j', 'i'))
        self.entPower.set_text(str(self.power))
        self.entMaxIter.set_text(str(self.maxIter))
        self.entBail.set_text(str(self.bail))
        
        # Run dialog
        response = self.dlg.run()
        self.dlg.hide()
        
        if (response < 0):
            return
        
        # Set new values
        try:
            self.const = complex(self.entConst.get_text().replace('i', 'j'))
        except:
            pass
        
        try:
            self.power = int(self.entPower.get_text())
        except:
            pass
        
        try:
            self.maxIter = int(self.entMaxIter.get_text())
        except:
            pass
        
        try:
            self.bail = int(self.entBail.get_text())
        except:
            pass

    def draw(self, window, progress, topLeft, bottomRight):
        '''
            This method renders the mandelbrot set to the given gtk.gdk.window1.
        '''
        # (wndXPos, wndYPos, xRes, yRes, bbs) = window.get_geometry()
        (xRes, yRes) = window.get_size()
        self.prepare_palette(window)
        
        delta = bottomRight - topLeft
        step  = complex(delta.real / xRes, delta.imag / yRes)
        startValue = topLeft - step
        ### const = topLeft - step
        
        progress.set_fraction(0.0)
        fracDelta = 1.0 / yRes
        frac = 0.0
        
        for y in range(yRes):
            frac += fracDelta
            if (frac > 1):
                frac = 1
            progress.set_fraction(frac)
            gtk.main_iteration(block=False)
            
            ### const = complex(topLeft.real - step.real, const.imag + step.imag)
            startValue = complex(topLeft.real - step.real, startValue.imag + step.imag)
            
            for x in range(xRes):
                ### const = complex(const.real + step.real, const.imag)
                startValue = complex(startValue.real + step.real, startValue.imag)
                
                z    = startValue
                z1   = startValue
                iter = 0
                
                while (iter < self.maxIter) and (abs(z) < self.bail):
                    iter += 1
                    z = z1
                    z1 = pow(z, self.power) + self.const
                
                color = self.palette[int(iter)]
                window.draw_point(color, x, y)
        
        progress.set_fraction(0.0)
