#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This module contains the mandelbrot fractal class.
'''

# Python modules
import gtk
import gtk.gdk
import math

# Application modules
from fractal import *

class Mandelbrot(Fractal):
    '''
        The mandelbrot fractal.
    '''
    
    def __init__(self, app):
        '''
            Constructor. Initializes the fractal calculation.
        '''
        Fractal.__init__(self, app)
        self.defaultTopLeft = -2 -1.25j
        self.defaultBottomRight = 1.2 + 1.25j

    def parameter_dialog(self):
        '''
            This method opens a GTK dialog and prompts the user for mandelbrot
            specific parameters.
        '''
        # Get object references
        self.gladeFile  = self.app.gladeFile
        self.wTree      = gtk.glade.XML(self.gladeFile, root="dlgMandelParam")
        self.dlg        = self.wTree.get_widget("dlgMandelParam")
        self.entMaxIter = self.wTree.get_widget("entMandelParamMaxIter")
        self.entBail    = self.wTree.get_widget("entMandelParamBail")
        
        self.dlg.connect("delete-event", self.dlg.hide_on_delete)
        self.dlg.set_icon_from_file(self.app.iconFile)
        
        # Set default values and display
        self.entMaxIter.set_text(str(self.maxIter))
        self.entBail.set_text(str(self.bail))
        
        # Run dialog
        response = self.dlg.run()
        self.dlg.hide()
        
        if (response < 0):
            return
        
        # Set new values
        try:
            self.maxIter = int(self.entMaxIter.get_text())
        except:
            pass
        
        try:
            self.bail = int(self.entBail.get_text())
        except:
            pass

    def draw(self, window, progress, topLeft, bottomRight):
        '''
            This method renders the mandelbrot set to the given gtk.gdk.window1.
        '''
        # (wndXPos, wndYPos, xRes, yRes, bbs) = window.get_geometry()
        (xRes, yRes) = window.get_size()
        self.prepare_palette(window)
        
        delta = bottomRight - topLeft
        step  = complex(delta.real / xRes, delta.imag / yRes)
        const = topLeft - step
        
        progress.set_fraction(0.0)
        fracDelta = 1.0 / yRes
        frac = 0.0
        
        for y in range(yRes):
            frac += fracDelta
            if (frac > 1):
                frac = 1
            progress.set_fraction(frac)
            gtk.main_iteration(block=False)
            
            const = complex(topLeft.real - step.real, const.imag + step.imag)
            
            for x in range(xRes):
                const = complex(const.real + step.real, const.imag)
                z    = 0 + 0j
                z1   = 0 + 0j
                iter = 0
                
                while (iter < self.maxIter) and (abs(z) < self.bail):
                    iter += 1
                    z = z1
                    z1 = pow(z, 2) + const
                
                color = self.palette[int(iter)]
                window.draw_point(color, x, y)
        
        progress.set_fraction(0.0)
