#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This dialog lets the user define an arbitary detail of the shown fractal
'''

# Python modules
import pygtk
import gtk
import gtk.glade
import gobject
pygtk.require('2.0')

class EditCorners:
    '''
        This dialog lets the user define an arbitary detail of the shown fractal
    '''

    def __init__(self, app=None):
        '''
            Constructor.
        '''
        if (not app):
            raise "Corner Edit Dialog needs the main application instance."
        
        # Store instance variables
        self.app = app
        
        self.gladeFile = self.app.gladeFile
        self.wTree           = gtk.glade.XML(self.gladeFile, root="dlgEditCorners")
        self.dlg             = self.wTree.get_widget("dlgEditCorners")
        self.btnOk           = self.wTree.get_widget("btnEditCornersOk")
        self.btnCanc         = self.wTree.get_widget("btnEditCornersCanc")
        self.btnFit          = self.wTree.get_widget("btnEditCornersFit")
        self.entScreenWidth  = self.wTree.get_widget("entEditCornersScreenWidth")
        self.entScreenHeight = self.wTree.get_widget("entEditCornersScreenHeight")
        self.entScreenRatio  = self.wTree.get_widget("entEditCornersScreenRatio")
        self.entFirst        = self.wTree.get_widget("entEditCornersFirst")
        self.entSecond       = self.wTree.get_widget("entEditCornersSecond")
        self.entRatio        = self.wTree.get_widget("entEditCornersRatio")
        
        self.btnFit.connect("clicked", self.adjust_fractal_ratio)
        
        self.screenWidth  = 0
        self.screenHeight = 0
        self.screenRatio  = 0
        self.firstCorner  = 0 + 0j
        self.secondCorner = 0 + 0j
        self.fractalRatio = 0

    def run(self):
        '''
            This method runs the dialog in a modal state. If the user pushes
            OK the new corners (topLeft and bottomRight) will be returned.
            Otherwise None will be returned
        '''
        if not self.app.wndMain.pixmap:
            return
        
        # Calculate screen size and aspect ratio
        self.screenWidth  = self.app.wndMain.pixmap.get_width()
        self.screenHeight = self.app.wndMain.pixmap.get_height()
        
        try:
            self.screenRatio = (self.screenWidth / 1.0) / (self.screenHeight / 1.0)
        except:
            screenRatio = 0.0
        
        # Calculate fractal size and aspect ratio
        self.firstCorner  = self.app.wndMain.topLeft
        self.secondCorner = self.app.wndMain.bottomRight
        
        try:
            self.fractalRatio = (abs(self.secondCorner.real - self.firstCorner.real) / 1.0) / (abs(self.secondCorner.imag - self.firstCorner.imag) / 1.0)
        except:
            self.fractalRatio = 0.0
        
        # Prefill entry fields
        self.update_entry_fields()
        
        # Show dialog
        response = self.dlg.run()
        self.dlg.hide()
        
        if (response < 0):
            return None
        
        # Get new corners
        self.firstCorner  = complex(self.entFirst.get_text().replace('i','j'))
        self.secondCorner = complex(self.entSecond.get_text().replace('i','j'))
        
        # Return new corners
        return (self.firstCorner, self.secondCorner)

    def update_entry_fields(self):
        '''
            This method refreshes the displayed values in the dialog.
        '''
        self.entScreenWidth.set_text(str(self.screenWidth))
        self.entScreenHeight.set_text(str(self.screenHeight))
        self.entScreenRatio.set_text(str(self.screenRatio))
        
        self.entFirst.set_text(str(self.firstCorner).replace('(', '').replace(')', '').replace('j', 'i'))
        self.entSecond.set_text(str(self.secondCorner).replace('(', '').replace(')', '').replace('j', 'i'))
        self.entRatio.set_text(str(self.fractalRatio))

    def adjust_fractal_ratio(self, widget=None, data=None):
        '''
            This method adjusts the fractal size so that its ratio matches
            the screen ratio
        '''
        # Calculate fractal size and ratio
        try:
            fractalWidth  = abs(self.secondCorner.real - self.firstCorner.real) / 1.0
            fractalHeight = abs(self.secondCorner.imag - self.firstCorner.imag) / 1.0
            fractalRatio  = fractalWidth / fractalHeight
        except:
            return
        
        # Caluculate new fractal ratio according to screen size
        if (fractalRatio == self.screenRatio):
            return
        
        if (fractalWidth < fractalHeight):
            newFractalWidth  = ((self.screenWidth / 1.0) / (self.screenHeight / 1.0)) * fractalHeight
            newFractalHeight = fractalHeight
        else:
            newFractalHeight = ((self.screenHeight / 1.0) / (self.screenWidth / 1.0)) * fractalWidth
            newFractalWidth  = fractalWidth
        
        # Scale fractal in order to enclose given corners
        if (newFractalWidth < fractalWidth):
            factor = newFractalWidth / fractalWidth
            newFractalWidth  = newFractalWidth  * factor
            newFractalHeight = newFractalHeight * factor
        if (newFractalHeight < fractalHeight):
            factor = newFractalHeight / fractalHeight
            newFractalWidth  = newFractalWidth  * factor
            newFractalHeight = newFractalHeight * factor
        
        self.fractalRatio = newFractalWidth / newFractalHeight
        
        # Calculate new corner points
        realOffset = (newFractalWidth - fractalWidth) / 2
        imagOffset = (newFractalHeight - fractalHeight) / 2
        
        re1 = min(self.firstCorner.real, self.secondCorner.real) - realOffset
        re2 = max(self.firstCorner.real, self.secondCorner.real) + realOffset
        im1 = min(self.firstCorner.imag, self.secondCorner.imag) - imagOffset
        im2 = max(self.firstCorner.imag, self.secondCorner.imag) + imagOffset
        
        self.firstCorner  = complex(re1, im1)
        self.secondCorner = complex(re2, im2)
        
        # Return calculated values
        self.update_entry_fields()
