#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This is the main window.
'''

# Python modules
import pygtk
import gtk
import gtk.glade
import gobject
import math
pygtk.require('2.0')

class Main:
    '''
        This is the main window.
    '''

    def __init__(self, app=None, fract=None):
        '''
            Constructor.   
        '''
        if (not app):
            raise "The main window needs a reference to the application object."
        
        # Store instance variables
        self.app   = app
        self.fract = fract
        
        self.gladeFile      = self.app.gladeFile
        self.wTree          = gtk.glade.XML(self.gladeFile, root="wndMain")
        self.wnd            = self.wTree.get_widget("wndMain")
        self.btnCalc        = self.wTree.get_widget("tbtnCalc")
        self.btnFracType    = self.wTree.get_widget("tbtnChooseFract")
        self.btnSaveImg     = self.wTree.get_widget("tbtnSaveImg")
        self.btnParam       = self.wTree.get_widget("tbtnParameters")
        self.btnClose       = self.wTree.get_widget("tbtnClose")
        self.btnRemove      = self.wTree.get_widget("btnRemove")
        self.btnMoveUp      = self.wTree.get_widget("btnUp")
        self.btnMoveDown    = self.wTree.get_widget("btnDown")
        self.drawArea       = self.wTree.get_widget("drawMain")
        self.tree           = self.wTree.get_widget("treeMain")
        self.btnNoCorner    = self.wTree.get_widget("btnNoCorners")
        self.btnEditCorners = self.wTree.get_widget("btnEditCorners")
        self.progress       = self.wTree.get_widget("pgbMain")
        
        self.drawArea.connect("expose-event", self.refresh_drawing_area)
        self.drawArea.connect("button-release-event", self.set_corner_point)
        self.btnNoCorner.connect("clicked", self.dont_draw_corners)
        self.btnEditCorners.connect("clicked", self.edit_corners)
        
        self.tree.connect("row-activated", self.get_corners_from_tree_activated)
        self.wnd.connect("delete-event", self.wnd.hide_on_delete)
        self.wnd.connect("hide", gtk.main_quit)
        
        self.wnd.set_icon_from_file(self.app.iconFile)
        self.wTree.signal_autoconnect(self)
        
        # Initialize list and drawing area
        self.init(redraw=False)

    def init(self, redraw=False):
        self.pixmap  = None
        self.pixmaps = []
        self.init_list()
        
        self.renderCount     = 0
        
        # Current selection in complex and in screen coordinates
        self.topLeft         = 0j
        self.bottomRight     = 9999j
        self.topLeftPx       = (0, 0)
        self.bottomRightPx   = (9999, 9999)
        
        # Coordinates of the currently shown fractal
        self.imgTopLeft      = 0j
        self.imgBottomRight  = 0j
        self.PixmapDx        = 0
        self.PixmapDy        = 0
        
        # Flags for setting corner points
        self.drawCorners     = False
        self.setBottomCorner = False
        
        if (redraw):
            self.clear_drawing_area()

    def init_list(self, addCols=True):
        '''
            This method initializes the history list.
        '''
        self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.tree.set_model(self.listStore)
        
        if (not addCols):
            return
        
        cell1 = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn("Top-Left", cell1, text=0)
        col1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col1.set_resizable(True)
        self.tree.append_column(col1)
        
        cell2 = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn("Bottom-Right", cell2, text=1)
        col2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col2.set_resizable(True)
        self.tree.append_column(col2)

    def get_corners_from_tree_activated(self, tree, path, col, data=None):
        '''
            This method handler will be called when the user activates a row in the
            list. This means if he double clicks on it or presses [ENTER]. In that
            case the selected image will be restored from memory and the corresponding
            corners will be set so the user can recalculate the picture if desired.
        '''
        entry = self.listStore[path]
        self.topLeft        = complex(str(entry[0]).replace('i', 'j'))
        self.bottomRight    = complex(str(entry[1]).replace('i', 'j'))
        self.imgTopLeft     = self.topLeft
        self.imgBottomRight = self.bottomRight
        self.pixmap         = self.pixmaps[path[0]]
        self.refresh_drawing_area()

    def dont_draw_corners(self, widget=None, data=None):
        '''
            This method removes the corner markers from the drawing area.
        '''
        self.drawCorners     = False
        self.setBottomCorner = False
        self.refresh_drawing_area()

    def edit_corners(self, widget=None, data=None):
        '''
            This method presents a dialog to edit the shown detail of the
            fractal.
        '''
        newCorners = self.app.dlgEditCorners.run()
        
        if not newCorners:
            return
        
        self.dont_draw_corners()
        
        re1 = newCorners[0].real
        re2 = newCorners[1].real
        im1 = newCorners[0].imag
        im2 = newCorners[1].imag
        
        self.topLeft = complex(min(re1, re2), min(im1, im2))
        self.bottomRight = complex(max(re1, re2), max(im1, im2))

    def set_corner_point(self, widget=None, event=None, data=None):
        '''
            This method sets the top-left and the bottom-right corner if the
            user clicks on the drawing area.
        '''
        if self.renderCount < 1:
            return False
        if (not self.pixmap):
            return False
        
        self.drawCorners = True
        
        # Store clicked point
        width = self.pixmap.get_width()
        height = self.pixmap.get_height()
        (x, y) = event.get_coords()
        x -= self.PixmapDx
        y -= self.PixmapDy
        
        re = ((x / width)  * (self.imgBottomRight.real - self.imgTopLeft.real)) + self.imgTopLeft.real
        im = ((y / height) * (self.imgBottomRight.imag - self.imgTopLeft.imag)) + self.imgTopLeft.imag
        point = complex(re, im)
        
        # Estimate relative distance to all four selection corners
        topLeft     = self.topLeft
        bottomLeft  = complex(self.topLeft.real, self.bottomRight.imag)
        topRight    = complex(self.bottomRight.real, self.topLeft.imag)
        bottomRight = self.bottomRight
        
        DistTopLeft     = math.sqrt(pow((point.real - topLeft.real), 2)     + pow((point.imag - topLeft.imag), 2))
        DistBottomLeft  = math.sqrt(pow((point.real - bottomLeft.real), 2)  + pow((point.imag - bottomLeft.imag), 2))
        DistTopRight    = math.sqrt(pow((point.real - topRight.real), 2)    + pow((point.imag - topRight.imag), 2))
        DistBottomRight = math.sqrt(pow((point.real - bottomRight.real), 2) + pow((point.imag - bottomRight.imag), 2))
        
        DistMin = min((DistTopLeft, DistBottomLeft, DistTopRight, DistBottomRight))
        
        # Store new coordinates relative to the closest corner
        if (DistMin == DistTopLeft):
            self.topLeft   = point
            self.topLeftPx = (x, y)
        
        elif (DistMin == DistBottomLeft):
            self.topLeft       = complex(point.real, self.topLeft.imag)
            self.bottomRight   = complex(self.bottomRight.real, point.imag)
            self.topLeftPx     = (x, self.topLeftPx[1])
            self.bottomRightPx = (self.bottomRightPx[0], y)
        
        elif (DistMin == DistTopRight):
            self.topLeft       = complex(self.topLeft.real, point.imag)
            self.bottomRight   = complex(point.real, self.bottomRight.imag)
            self.topLeftPx     = (self.topLeftPx[0], y)
            self.bottomRightPx = (x, self.bottomRightPx[1])
        
        elif (DistMin == DistBottomRight):
            self.bottomRight   = point
            self.bottomRightPx = (x, y)
        
        # Redraw drawing area with corner markers
        self.refresh_drawing_area()
        return False

    def clear_drawing_area(self):
        '''
            This method clears the drawing area to white.
        '''
        (xPos, yPos, width, height, bits) = self.drawArea.window.get_geometry()
        white = gtk.gdk.Color(pixel=0xFFFFFFFF)
        gc = gtk.gdk.GC(self.drawArea.window, white, white, function=gtk.gdk.COPY, fill=gtk.gdk.SOLID)
        self.drawArea.window.draw_rectangle(gc, True, 0, 0, width, height)

    def refresh_drawing_area(self, widget=None, event=None, data=None):
        '''
            This method redraws the drawing area from the internal buffer.
        '''
        
        # Fill drawing area white
        self.clear_drawing_area()
        
        # Blit current pixbufer into center
        if (not self.pixmap):
            return
        
        (xPos, yPos, width, height, bits) = self.drawArea.window.get_geometry()
        wd = self.pixmap.get_width()
        hg = self.pixmap.get_height()
        dx = (width - wd) / 2
        dy = (height - hg) / 2
        self.PixmapDx = dx
        self.PixmapDy = dy
        sx = 0
        sy = 0
        
        if (dx < 0):
            sx -= dx
            wd += (2 * dx)
            dx = 0
        if (dy < 0):
            sy -= dy
            hg += (2 * dy)
            dy = 0
        
        self.drawArea.window.draw_pixbuf(None, self.pixmap, sx, sy, dx, dy, wd, hg, gtk.gdk.RGB_DITHER_NORMAL, 0, 0)
        
        # Draw corner selection markers
        if (not self.drawCorners):
            return
        
        x  = self.topLeftPx[0]
        y  = self.topLeftPx[1]
        wd = abs(self.bottomRightPx[0] - x)
        hg = abs(self.bottomRightPx[1] - y)
        x += self.PixmapDx
        y += self.PixmapDy
        
        if (x < 0):
            wd += x
            x   = 0
        if (y < 0):
            hg += y
            y   = 0
        
        wd = min(wd, width - x)
        hg = min(hg, height - y)
        
        cl = gtk.gdk.Color(pixel=0xFFFFFFFF)
        gc = self.drawArea.window.new_gc(foreground=cl, background=cl, fill=gtk.gdk.SOLID, function=gtk.gdk.XOR, graphics_exposures=False)
        
        self.drawArea.window.draw_rectangle(gc, True, x, y, wd, hg)

    def run(self):
        '''
            This method shows the main window and runs the GTK main loop.
        '''
        self.wnd.show()
        self.clear_drawing_area()
        gtk.main()

    def check_fract_object(self):
        '''
            This method checks whether a fractal object exists or not. If none
            exists the user will be promptes to select one.
        '''
        if (self.fract):
            return
        
        self.on_tBtnChooseFract_clicked()

    def on_tbtnCalc_clicked(self, widget=None, data=None):
        '''
            Calculation button signal handler. This method causes the fractal to be
            recalculated and displayed.
        '''
        self.check_fract_object()
        if (not self.fract):
            return
        
        # Create a new pixel buffer to render the fractal into
        (xPos, yPos, width, height, bits) = self.drawArea.window.get_geometry()
        self.pixmap = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
        self.pixmap.fill(0xFFFFFFFF)     # RGBA: White
        
        # Let the fractal object render the fractal
        if (self.renderCount < 1):
            self.topLeft     = self.fract.defaultTopLeft
            self.bottomRight = self.fract.defaultBottomRight
        
        self.fract.draw(self.drawArea.window, self.progress, self.topLeft, self.bottomRight)
        self.renderCount += 1
        
        # Store the rendered fractal in the internal redraw buffer
        self.pixmap = self.pixmap.get_from_drawable(self.drawArea.window, gtk.gdk.colormap_get_system(), 0, 0, 0, 0, width, height)
        self.imgTopLeft     = self.topLeft
        self.imgBottomRight = self.bottomRight
        self.dont_draw_corners()
        
        # Apend fractal to the list if it differs from the last on
        nr = len(self.listStore) - 1
        if (nr >= 0):
            le = self.listStore[nr]
            tl = complex(str(le[0]).replace('i','j'))
            br = complex(str(le[1]).replace('i','j'))
            
            if (tl != self.topLeft) or (br != self.bottomRight):
                append = True
            else:
                append = False
        else:
            append = True
        
        if append:
            self.pixmaps.append(self.pixmap)
            tl = str(self.topLeft).replace('(', '').replace(')', '').replace('j', 'i')
            br = str(self.bottomRight).replace('(', '').replace(')', '').replace('j', 'i')
            self.listStore.append([tl, br])
        else:
            self.pixmaps[nr] = self.pixmap

    def on_tBtnChooseFract_clicked(self, widget=None, data=None):
        '''
            Choose fractal signal handler. This method opens a dialog in which the
            user can change the current fractal type.
        '''
        fractalObject = self.app.dlgFractType.run()
        if (not fractalObject):
            return
        
        fractalObject.parameter_dialog()
        
        self.fract = fractalObject
        self.init(redraw=True)

    def on_tbtnSaveImg_clicked(self, widget=None, data=None):
        '''
            Save image signal handler. This method queries the user for a filename
            and an image resolution. Then it causes an image to be saved for the
            currently displayed fractal.
        '''
        self.check_fract_object()
        if (not self.fract):
            return
        
        # Request filename from user
        fileChooser = gtk.FileChooserDialog(title="Save Image", parent=self.wnd, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL, -1, gtk.STOCK_OK, 0))
        response = fileChooser.run()
        fileName = fileChooser.get_filename()
        fileChooser.hide()
        
        if (response < 0):
            return
        if (not fileName):
            return
        
        # Request resolution from user
        response = self.app.dlgImageResolution.run()
        if (not response):
            return
        
        width = response[0]
        height = response[1]
        
        # Create a new window to render the fractal into
        drawable = gtk.gdk.Pixmap(self.drawArea.window, width, height, -1)
        pixmap = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
        pixmap.fill(0xFFFFFFFF)     # RGBA: White
        
        # Let the fractal object render the fractal
        if (self.renderCount < 1):
            self.topLeft     = self.fract.defaultTopLeft
            self.bottomRight = self.fract.defaultBottomRight
        
        self.fract.draw(drawable, self.progress, self.topLeft, self.bottomRight)
        self.renderCount += 1
        
        # Store the rendered fractal to a png file
        pixmap = pixmap.get_from_drawable(drawable, gtk.gdk.colormap_get_system(), 0, 0, 0, 0, width, height)
        pixmap.save(fileName, "png", {})

    def on_tbtnParameters_clicked(self, widget=None, data=None):
        '''
            This method checks for a selected fractal and opens it parameter
            dialog. If the fractal doesn't override fractal.parameter_dialog
            no dialog will be shown.
        '''
        self.check_fract_object()
        
        if (self.fract):
            self.fract.parameter_dialog()

    def on_tbtnClose_clicked(self, widget=None, data=None):
        '''
            Signal handler for click on close button. Closes the main window
            and thus exits the application.
        '''
        self.wnd.hide()

    def on_btnRemove_clicked(self, widget=None, data=None):
        '''
            Remove item from list signal handler. Removes the currently selected
            item from the list.
        '''
        sel = self.tree.get_selection()
        (model, iter) = sel.get_selected()
        
        if (not iter):
            return
        
        path = self.listStore.get_path(iter)
        nr = int(path[0])
        
        self.listStore.remove(iter)
        self.pixmaps.remove(self.pixmaps[nr])

    def on_btnUp_clicked(self, widget=None, data=None):
        '''
            Move item up signal handler. This method moves the currently selected
            item in the list one position upwards.
        '''
        sel = self.tree.get_selection()
        (model, iter) = sel.get_selected()
        
        if (not iter):
            return
        
        path = self.listStore.get_path(iter)
        nr = int(path[0])
        
        if (nr == 0):
            return
            
        try:
            iter1 = self.listStore.get_iter(nr - 1)
        except:
            iter1 = None
        
        if (not iter1):
            return
        
        self.listStore.swap(iter, iter1)
        
        pix1 = self.pixmaps[nr]
        pix2 = self.pixmaps[nr - 1]
        self.pixmaps[nr] = pix2
        self.pixmaps[nr - 1] = pix1

    def on_btnDown_clicked(self, widget=None, data=None):
        '''
            Move item down signal handler. This method moves the currently selected
            item in the list one position downwards.
        '''
        sel = self.tree.get_selection()
        (model, iter) = sel.get_selected()
        
        if (not iter):
            return
        
        path = self.listStore.get_path(iter)
        nr = int(path[0])
        
        try:
            iter1 = self.listStore.get_iter(nr + 1)
        except:
            iter1 = None
        
        if (not iter1):
            return
        
        self.listStore.swap(iter, iter1)
        
        pix1 = self.pixmaps[nr]
        pix2 = self.pixmaps[nr + 1]
        self.pixmaps[nr] = pix2
        self.pixmaps[nr + 1] = pix1
