#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This is the main module. It starts the program and opens the
    main window.
'''

# Python modules
import os
import os.path
import optparse
import ConfigParser

# Application modules
import gui.wnd.main
import gui.dlg.imageResolution
import gui.dlg.animation
import gui.dlg.fractType
import gui.dlg.editCorners

import fractal.mandelbrot
import fractal.julia

class FractGTK:
    '''
        This is the main class. It holds application wide variables and
        methods.
    '''
    
    def __init__(self):
        '''
            Constructor: Initializes the application and opens the main window.
        '''
        # Application wide variables
        self.appname   = "FractGTK"
        self.applong   = "A minimal GTK fractal browser"
        self.version   = "0.1"
        self.year      = "2007"
        self.author    = "Dennis Schulmeister <dennis@windows3.de>"
        self.copyright = "(C) %s %s" % (self.year, self.author)
        self.appPath   = os.path.abspath("./")
        self.gladeFile = os.path.abspath("./gui.glade")
        self.iconFile  = os.path.abspath("./icon.png")
        
        # Parse command line arguments
        self.argParser = optparse.OptionParser()
        
        self.argParser.add_option('-v', '--version', action='store_true', dest='showVersion', help='Display program version')
        self.argParser.set_defaults(showVersion = False)
        
        (self.options, self.args) = self.argParser.parse_args()
        showVersion  = self.options.showVersion
        
        # Show version number and quit if requested
        if (showVersion):
            self.show_version()
            return
        
        # Build a list of all available fractals
        self.fractals = {
                            'Mandelbrot': fractal.mandelbrot.Mandelbrot,
                            'Julia':      fractal.julia.Julia
                        }
        
        # Application wide objects
        self.dlgAnimation = gui.dlg.animation.Animation()
        self.dlgImageResolution = gui.dlg.imageResolution.ImageResolution(app=self)
        self.dlgFractType = gui.dlg.fractType.FractalType(app=self, dict=self.fractals)
        self.dlgEditCorners = gui.dlg.editCorners.EditCorners(app=self)
        
        # Ask the user which fractal to start with
        fractalObject = self.dlgFractType.run()
        if (not fractalObject):
            return
        fractalObject.parameter_dialog()
        
        # Show the main window
        self.wndMain = gui.wnd.main.Main(app=self, fract=fractalObject)
        self.wndMain.run()

    def show_version(self):
        '''
            This method prints some version information to StdOut.
        '''
        versionString  = '%s - %s\n' % (self.appname, self.applong)
        versionString += '%s\n' % (self.copyright)
        versionString += '\n'
        versionString += 'This is free software. You may redistribute copies of it under the terms of\n'
        versionString += 'the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.\n'
        versionString += 'There is NO WARRANTY, to the extent permitted by law.\n'
        
        print(versionString)

if (__name__ == '__main__'):
    main = FractGTK()
